@@ -17,7 +17,7 @@ CStreamProtocolHandler::~CStreamProtocolHandler()
 
 void CStreamProtocolHandler::Start()
 {
-   m_vlcPlayer->Play(m_url, m_vlcoptions);
+   m_vlcPlayer->Play(elURI, m_url, m_vlcoptions);
 
 }
 
