@@ -8,6 +8,7 @@
 #include "configuration.h"
 #include "userdataobject.h"
 #include "proxy.h"
+#include "main.h"
 
 using namespace tvlite;
 
@@ -24,8 +25,11 @@ CVlcPlayer::CVlcPlayer(wxEvtHandler *parent) :
    m_wasStopped(false),
    m_rState(rsNone),
    m_volume(80),
+   m_locationType(elInvalid),
+   m_fileDescriptor(-1),
+   m_cb_data(NULL),
+   m_questionDlg(NULL),
    m_pparent(parent)
-
 {
    m_trackMap[etVideo] = -1;
    m_trackMap[etAudio] = -1;
@@ -36,6 +40,8 @@ CVlcPlayer::~CVlcPlayer()
 {
    if (m_vlc_inst) libvlc_release(m_vlc_inst);
    if (m_media_player) libvlc_media_player_release(m_media_player);
+   if (m_questionDlg != NULL) m_questionDlg->Destroy();
+   
 }
 
 
@@ -67,6 +73,7 @@ void CVlcPlayer::Init(wxWindow *wnd)
         libvlc_media_player_set_hwnd(m_media_player, wnd->GetHandle());
         m_wnd = wnd;
     #endif
+    //attach events
     libvlc_event_attach( m_vlc_evt_man, libvlc_MediaPlayerBuffering,   VlcEvtCallback, this );
     libvlc_event_attach( m_vlc_evt_man, libvlc_MediaPlayerPlaying,         VlcEvtCallback, this );
     libvlc_event_attach( m_vlc_evt_man, libvlc_MediaPlayerPaused,          VlcEvtCallback, this );
@@ -88,6 +95,17 @@ void CVlcPlayer::Init(wxWindow *wnd)
     libvlc_event_attach( m_vlc_evt_man, libvlc_MediaPlayerAudioVolume, VlcEvtCallback, this );
 
     libvlc_audio_set_volume(m_media_player,80);
+    //set dialog callbacks
+    m_dialogCallbacks.pf_display_error = NULL;
+    m_dialogCallbacks.pf_display_question = DisplayQuestion;
+    m_dialogCallbacks.pf_cancel = DialogCancel;
+    m_dialogCallbacks.pf_display_login = NULL;
+    m_dialogCallbacks.pf_display_progress = NULL;
+    m_dialogCallbacks.pf_update_progress =NULL; 
+    //m_questionDlg = new CVLCQuestionDialog((wxWindow*)wxGetApp().GetMainFrame());
+
+    libvlc_dialog_set_callbacks(m_vlc_inst, &m_dialogCallbacks, (void*)this);
+
 
 }
 
@@ -101,116 +119,133 @@ void CVlcPlayer::SetFull()
 {
         libvlc_set_fullscreen	(m_media_player, true);
 }
-
-void CVlcPlayer::Play(wxString location, wxArrayString options)
+void CVlcPlayer::Play(ELocationType type, int fd, wxArrayString options)
+{
+      m_fileDescriptor = fd;
+      Play(type, wxString(""), options);
+}
+void CVlcPlayer::Play(ELocationType type, wxString location, wxArrayString options)
 {
         DBG_INFO("Entering Play");
         m_wasStopped = false;
-        if (location == "")
+        if (type == elURI && location == "")
         {
            return;
         }
         m_location = location;
         m_vlcoptions = options;
         //TODO uri disseminate
-        libvlc_media_t *media;
-        media = libvlc_media_new_location(m_vlc_inst, (const char*)location.utf8_str());
-        wxURI uri(location);
-        wxString host = uri.GetServer();
-        DBG_INFO("Host is %s", (const char*)host.utf8_str());
-        bool islocalAddress = (host.Find("127.0.0.1") != wxNOT_FOUND ||
-                               host.Find("localhost") != wxNOT_FOUND ||
-                               host.Find("::1") != wxNOT_FOUND) ? true : false;
-
-        CProxy *proxy = CProxy::GetConfiguredProxy();
-        if (proxy && (!islocalAddress))
+        libvlc_media_t *media = nullptr;
+        
+        if (type == elURI)
         {
-           wxString vlcProxyOption;
-           if (proxy->GetProxyType() == E_PROXY_HTTP)
+           media = libvlc_media_new_location(m_vlc_inst, (const char*)location.utf8_str());
+           wxURI uri(location);
+           wxString host = uri.GetServer();
+           DBG_INFO("Host is %s", (const char*)host.utf8_str());
+           bool islocalAddress = (host.Find("127.0.0.1") != wxNOT_FOUND ||
+                                  host.Find("localhost") != wxNOT_FOUND ||
+                                  host.Find("::1") != wxNOT_FOUND) ? true : false;
+
+           CProxy *proxy = CProxy::GetConfiguredProxy();
+           if (proxy && (!islocalAddress))
            {
-              vlcProxyOption = "http-proxy=" + proxy->GetVlcURL();
-              DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
-              libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
-              if (proxy->IsProxyAuthUsed())
+              wxString vlcProxyOption;
+              if (proxy->GetProxyType() == E_PROXY_HTTP)
               {
-                 vlcProxyOption = "http-proxy-pwd=" + proxy->GetProxyPassword();
+                 vlcProxyOption = "http-proxy=" + proxy->GetVlcURL();
                  DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
                  libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
+                 if (proxy->IsProxyAuthUsed())
+                 {
+                    vlcProxyOption = "http-proxy-pwd=" + proxy->GetProxyPassword();
+                    DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
+                    libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
+                 }
               }
-           }
-           else if (proxy->GetProxyType() == E_PROXY_SOCKS)
-           {
-              vlcProxyOption = "socks=" + proxy->GetVlcURL();
-              DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
-              libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
-              if (proxy->IsProxyAuthUsed())
+              else if (proxy->GetProxyType() == E_PROXY_SOCKS)
               {
-                 vlcProxyOption = "socks-user=" + proxy->GetProxyUserName();
-                 DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
-                 libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
-                 vlcProxyOption = "socks-pwd=" + proxy->GetProxyPassword();
+                 vlcProxyOption = "socks=" + proxy->GetVlcURL();
                  DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
                  libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
+                 if (proxy->IsProxyAuthUsed())
+                 {
+                    vlcProxyOption = "socks-user=" + proxy->GetProxyUserName();
+                    DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
+                    libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
+                    vlcProxyOption = "socks-pwd=" + proxy->GetProxyPassword();
+                    DBG_INFO("Setting VLC option: %s", (const char*)vlcProxyOption.utf8_str());
+                    libvlc_media_add_option(media, (const char*)vlcProxyOption.utf8_str());
+                 }
               }
+              delete proxy;
            }
-           delete proxy;
-        }
-        bool addUserAgent = true;
-        if (options.Count() != 0)
-        {
-          
-           for (size_t i = 0; i < options.Count(); i++)
+           bool addUserAgent = true;
+           if (options.Count() != 0)
            {
-              DBG_INFO("Setting VLC option: %s", (const char*)options[i].utf8_str());
-              if (options[i].Matches("*http-user-agent*"))
+             
+              for (size_t i = 0; i < options.Count(); i++)
               {
-                  addUserAgent = false;
+                 DBG_INFO("Setting VLC option: %s", (const char*)options[i].utf8_str());
+                 if (options[i].Matches("*http-user-agent*"))
+                 {
+                     addUserAgent = false;
+                 }
+                 libvlc_media_add_option(media, (const char*)options[i].utf8_str());
               }
-              libvlc_media_add_option(media, (const char*)options[i].utf8_str());
+           }
+           if (addUserAgent)
+           {
+             wxString userAgentOption;
+             userAgentOption << "http-user-agent=" << GetUserAgent() ;
+             libvlc_media_add_option(media, (const char*)userAgentOption.utf8_str());
            }
         }
-        if (addUserAgent)
+        else if (type == elFileDescriptor)
         {
-          wxString userAgentOption;
-          userAgentOption << "http-user-agent=" << GetUserAgent() ;
-          libvlc_media_add_option(media, (const char*)userAgentOption.utf8_str());
-        }
-
-        libvlc_video_set_mouse_input(m_media_player, FALSE);
-        libvlc_media_player_set_media(m_media_player, media);
-        libvlc_media_release(media);
-//for playing (rendering)
+           if (m_fileDescriptor != -1)
+           media = libvlc_media_new_fd(m_vlc_inst, m_fileDescriptor);
+        }  
+        else
         {
-           wxString renderer = CConfiguration::Instance()->GetVLCConfiguration().GetRender();
-           wxString options = ":avcodec-hw=" + renderer;
-           libvlc_media_add_option(media,(const char*)options.utf8_str());
+           //invalid location -> media will be invalid
         }
-//for recording
-        if (m_rState == rsFile)
-        {
-           wxString options = ":sout=#duplicate{dst=display,dst=std{access=file,mux=ts,dst="  + m_recordFileName + "}" ;
-           libvlc_media_add_option(media,(const char*)options.utf8_str());
+//Now that we're all set, let's play the media        
+        if (media != nullptr)
+        { 
+           libvlc_video_set_mouse_input(m_media_player, FALSE);
+           libvlc_media_player_set_media(m_media_player, media);
+           libvlc_media_release(media);
+   //for playing (rendering)
+           {
+              wxString renderer = CConfiguration::Instance()->GetVLCConfiguration().GetRender();
+              wxString options = ":avcodec-hw=" + renderer;
+              libvlc_media_add_option(media,(const char*)options.utf8_str());
+           }
+   //for recording
+           if (m_rState == rsFile)
+           {
+              wxString options = ":sout=#duplicate{dst=display,dst=std{access=file,mux=ts,dst="  + m_recordFileName + "}" ;
+              libvlc_media_add_option(media,(const char*)options.utf8_str());
+           }
+   //for transmitting
+           else if (m_rState == rsStream)
+           {
+               wxString streamPort = wxString::Format("%d",m_streamPort);
+               wxString options = ":sout=#duplicate{dst=display,dst=std{access=http,mux=ts,dst=:" + streamPort + "}";
+               libvlc_media_add_option (media,(const char*)options.utf8_str());
+           }
+           libvlc_audio_set_volume(m_media_player,m_volume);
+           int  __attribute__((unused)) rv = libvlc_media_parse_with_options(media, libvlc_media_fetch_network , -1);
+           DBG_INFO("Parse rv = %d", rv);
+           libvlc_media_player_play(m_media_player);
         }
-//for transmitting
-        else if (m_rState == rsStream)
-        {
-            wxString streamPort = wxString::Format("%d",m_streamPort);
-            wxString options = ":sout=#duplicate{dst=display,dst=std{access=http,mux=ts,dst=:" + streamPort + "}";
-            libvlc_media_add_option (media,(const char*)options.utf8_str());
-        }
-        libvlc_audio_set_volume(m_media_player,m_volume);
-        int  __attribute__((unused)) rv = libvlc_media_parse_with_options(media, libvlc_media_fetch_network , -1);
-        DBG_INFO("Parse rv = %d", rv);
-        libvlc_media_player_play(m_media_player);
-
-
-
 }
 
 void CVlcPlayer::Play()
 {
    ResetTracks();
-   Play(m_location, m_vlcoptions);
+   Play(m_locationType, m_location, m_vlcoptions);
 }
 
 void CVlcPlayer::Stop()
@@ -612,3 +647,68 @@ libvlc_media_player_t *CVlcPlayer::GetMediaPlayer()
 {
    return m_media_player;
 }
+
+
+
+void CVlcPlayer::QuestionDlgShow (std::shared_ptr<SDlgData> data)
+{
+    std::shared_ptr<SDlgData> d = data;
+    if (m_questionDlg != NULL)
+    {
+        m_questionDlg->Destroy();
+        m_questionDlg = NULL;
+    }   
+    m_questionDlg = new CVLCQuestionDialog((wxWindow*)wxGetApp().GetMainFrame());
+    if (m_questionDlg != NULL)
+    {
+       
+       m_questionDlg->SetData(d);
+       int exitCode = m_questionDlg->ShowModal();
+       int action = m_questionDlg->GetAction();
+       int rc = 0;
+       if (exitCode == wxID_OK)
+       {
+          rc = libvlc_dialog_post_action (d->m_p_id, action);
+       }
+       else if (exitCode == wxID_CANCEL)
+       {
+          rc = libvlc_dialog_dismiss(d->m_p_id);
+       }
+       else
+       {
+          //nothing to do
+       }
+       if (rc != 0)
+       {
+           wxMessageBox(_("Internal VLC error"), _("Error"), wxOK|wxICON_EXCLAMATION, m_questionDlg);
+       }
+       m_questionDlg->Destroy();
+       m_questionDlg = NULL;  
+    }                            
+}
+
+void CVlcPlayer::DisplayQuestion(void *p_data, libvlc_dialog_id *p_id, const char *psz_title, const char *psz_text, 
+                               libvlc_dialog_question_type i_type,
+                               const char *psz_cancel, const char *psz_action1, const char *psz_action2)
+{
+  
+   CVlcPlayer *pVlcplayer = (CVlcPlayer *)p_data;
+   if (pVlcplayer != NULL)
+   {
+        std::shared_ptr<SDlgData> data = std::make_shared<SDlgData>(p_id, psz_title, psz_text, i_type, psz_cancel, psz_action1, psz_action2);
+        pVlcplayer->CallAfter(&CVlcPlayer::QuestionDlgShow, data);
+   }
+}
+
+void CVlcPlayer::DisplayError(void *p_data, const char *psz_title, const char *psz_text)
+{
+   wxWindow *parent = (wxWindow*)wxGetApp().GetMainFrame(); 
+   wxMessageBox(wxString::FromUTF8(psz_text), wxString::FromUTF8(psz_title), wxOK|wxICON_EXCLAMATION, parent);
+}
+
+void CVlcPlayer::DialogCancel(void *p_data, libvlc_dialog_id *p_id)
+{
+   DBG_INFO("The dialog was cancelled"); 
+   libvlc_dialog_dismiss(p_id);
+}
+
